from cargo2rpm.metadata import Metadata, Package


def guess_main_package(metadata: Metadata) -> Package:
    if not metadata.is_workspace():
        return metadata.packages[0]

    with_bin = []
    with_cdylib = []

    for package in metadata.packages:
        for target in package.targets:
            if "bin" in target.kind and "bin" in target.crate_types:
                with_bin.append(package)
            if "cdylib" in target.kind and "cdylib" in target.crate_types:
                with_cdylib.append(package)

    if len(with_cdylib) == 1:
        return with_cdylib[0]

    if len(with_bin) == 1:
        return with_bin[0]

    raise ValueError("Heuristic for determining the main crate of a cargo workspace failed. Please report a bug.")


def package_uses_rust_1_60_feature_syntax(features: dict[str, list[str]]) -> bool:
    for (name, deps) in features.items():
        for dep in deps:
            if "?/" in dep:
                return True

            if dep.startswith("dep:"):
                if len(deps) != 1:
                    return True
                if dep.removeprefix("dep:") != name:
                    return True

    return False
